@@ -1,8 +1,8 @@
 /*
-  $Id: document.cxx 14474 2012-03-19 13:04:14Z sloot $
+  $Id: document.cxx 15909 2013-04-03 13:52:24Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/libfolia/trunk/src/document.cxx $
 
-  Copyright (c) 1998 - 2012
+  Copyright (c) 1998 - 2013
   ILK   - Tilburg University
   CLiPS - University of Antwerp
  
@@ -37,15 +37,16 @@
 #include <map>
 #include <algorithm>
 #include <stdexcept>
-#include "folia/document.h"
-#include "folia/folia.h"
+#include "ticcutils/XMLtools.h"
+#include "libfolia/document.h"
 #include "config.h"
 
 using namespace std;
+using namespace TiCC;
 
 namespace folia {
 
-  const string FOLIAVERSION = "0.8"; // the FoLiA version we implement
+  const string FOLIAVERSION = "0.8.2"; // the FoLiA version we implement
   const string NSFOLIA = "http://ilk.uvt.nl/folia";
   const string NSDCOI = "http://lands.let.ru.nl/projects/d-coi/ns/1.0";
   const string NSIMDI = "http://www.mpi.nl/IMDI/Schema/IMDI";
@@ -66,7 +67,8 @@ namespace folia {
     metadata = 0;
     xmldoc = 0;
     foliadoc = 0;
-    _foliaNs = 0;
+    _foliaNsIn = 0;
+    _foliaNsOut = 0;
     debug = 0;
     version = FOLIAVERSION;
   }
@@ -126,14 +128,20 @@ namespace folia {
     else
       loadall = true;
     it = kwargs.find( "version" );
-    if ( it != kwargs.end() )
+    if ( it != kwargs.end() ){
       version = it->second;
+    }
     else
       version.clear();
-
+    
     it = kwargs.find( "id" );
     if ( it != kwargs.end() ){
-      _id = it->second;
+      if ( isNCName( it->second ) ){
+	_id = it->second;
+      }
+      else {
+	// isNCName throws
+      }
       happy = true;
     }
     else {
@@ -163,13 +171,27 @@ namespace folia {
       throw DuplicateIDError( s );
   }
 
+  static int error_sink(void *mydata, xmlError *error ){
+    int *cnt = (int*)mydata;
+    if ( *cnt == 0 ){
+      cerr << "\nXML-error: " << error->message << endl;
+    }
+    (*cnt)++;
+    return 1;
+  }
+
   bool Document::readFromFile( const string& s ){
     if ( xmldoc ){
       throw runtime_error( "Document is aready initialized" );
       return false;
     }
-    xmldoc = xmlReadFile( s.c_str(), 0, 0 );
+    int cnt = 0;
+    xmlSetStructuredErrorFunc( &cnt, (xmlStructuredErrorFunc)error_sink );
+    xmldoc = xmlReadFile( s.c_str(), 0, XML_PARSE_NOBLANKS );
     if ( xmldoc ){
+      if ( cnt > 0 ){
+	throw XmlError( "document is invalid" );
+      }
       if ( debug )
 	cout << "read a doc from " << s << endl;
       foliadoc = parseXml();
@@ -192,8 +214,13 @@ namespace folia {
       throw runtime_error( "Document is aready initialized" );
       return false;
     }
-    xmldoc = xmlReadMemory( s.c_str(), s.length(), 0, 0, 0 );
+    int cnt = 0;
+    xmlSetStructuredErrorFunc( &cnt, (xmlStructuredErrorFunc)error_sink );
+    xmldoc = xmlReadMemory( s.c_str(), s.length(), 0, 0, XML_PARSE_NOBLANKS );
     if ( xmldoc ){
+      if ( cnt > 0 ){
+	throw XmlError( "document is invalid" );
+      }
       if ( debug )
 	cout << "read a doc from string" << endl;
       foliadoc = parseXml();
@@ -217,12 +244,16 @@ namespace folia {
     return os;
   }
 
-  bool Document::save( const string& fn, const string& nsLabel ) {
+  bool Document::save( ostream& os, const string& nsLabel, bool kanon ) {
+    string s = toXml( nsLabel, kanon );
+    os << s << endl;
+    return os;
+  }
+
+  bool Document::save( const string& fn, const string& nsLabel, bool kanon ) {
     ofstream os( fn.c_str() );
     if ( os.good() ) {
-      string s = toXml( nsLabel );
-      os << s << endl;
-      return true;
+      return save( os, nsLabel, kanon );
     }
     throw runtime_error( "saving to file " + fn + " failed" );
     return false;
@@ -266,6 +297,12 @@ namespace folia {
     return foliadoc->select<Sentence>( excludeSet );
   }
   
+  vector<Sentence*> Document::sentenceParts() const {
+    static set<ElementType> excludeSet;
+    vector<Sentence*> sents = foliadoc->select<Sentence>( excludeSet );
+    return sents;
+  }
+  
   Sentence *Document::sentences( size_t index ) const {
     vector<Sentence*> v = sentences();
     if ( index < v.size() ){
@@ -292,9 +329,13 @@ namespace folia {
       excludeSet.insert( Alternative_t );
       excludeSet.insert( Chunk_t );
       excludeSet.insert( SyntacticUnit_t );
+      excludeSet.insert( Coreferences_t );
+      excludeSet.insert( Semroles_t );
       excludeSet.insert( Entity_t );
-      excludeSet.insert( DependencyHead_t );
+      excludeSet.insert( Headwords_t );
+      excludeSet.insert( TimingLayer_t );
       excludeSet.insert( DependencyDependent_t );
+      excludeSet.insert( TimeSegment_t );
     }
     return foliadoc->select<Word>( excludeSet );
   }
@@ -378,6 +419,7 @@ namespace folia {
 	string s;
 	string a;
 	string t;
+	string d;
 	KWargs::const_iterator it = att.find("set" );
 	if ( it != att.end() ){
 	  s = it->second;
@@ -392,7 +434,11 @@ namespace folia {
 	if ( it != att.end() ){
 	  t = it->second;
 	}
-	declare( type, s, a, t );
+	it = att.find( "datetime" );
+	if ( it != att.end() ){
+	  d = parseDate( it->second );
+	}
+	declare( type, s, a, t, d );
       }
       n = n->next;
     }
@@ -414,6 +460,7 @@ namespace folia {
       throw XmlError("FoLiA Document has no ID!");
       return 0;
     }
+    setAttributes( att );
     FoliaElement *result = FoliaElement::createElement( this, Name(root) );
     if ( debug > 2 )
       cerr << "created " << root << endl;
@@ -480,12 +527,54 @@ namespace folia {
     }
     return result;
   }
+  
+  void Document::addStyle( const std::string& type, const std::string& href ){
+    if ( type == "text/xsl" ){
+      multimap<string,string>::iterator it = styles.find( type );
+      if ( it != styles.end() )
+	throw XmlError( "multiple 'text/xsl' style-sheets defined." );
+    }
+    styles.insert( make_pair( type, href ) );
+  }
+  
+  void Document::replaceStyle( const std::string& type, 
+			       const std::string& href ){
+    multimap<string,string>::iterator it = styles.find( type );
+    if ( it != styles.end() ){
+      it->second = href;
+    }
+    else {
+      styles.insert( make_pair( type, href ) );
+    }
+  }
 
   void Document::getstyles(){
     xmlNode *pnt = xmldoc->children;
     while ( pnt ){
       if ( pnt->type == XML_PI_NODE && Name(pnt) == "xml-stylesheet" ){
-	addStyle( (const char*)pnt->content );
+	string content = (const char*)pnt->content;
+	string type;
+	string href;
+	vector<string> v;
+	split( content, v );
+	if ( v.size() == 2 ){
+	  vector<string> w;
+	  split_at( v[0], w, "=" );
+	  if ( w.size() == 2 && w[0] == "type" ){
+	    type = w[1].substr(1,w[1].length()-2);
+	  }
+	  w.clear();
+	  split_at( v[1], w, "=" );
+	  if ( w.size() == 2 && w[0] == "href" ){
+	    href = w[1].substr(1,w[1].length()-2);
+	  }
+	}
+	if ( !type.empty() && !href.empty() ){
+	  addStyle( type, href );
+	}
+	else {
+	  throw XmlError( "problem parsing line: " + content );
+	}
       }
       pnt = pnt->next;
     }
@@ -494,11 +583,13 @@ namespace folia {
   FoliaElement* Document::parseXml( ){
     getstyles();
     xmlNode *root = xmlDocGetRootElement( xmldoc );
+    _foliaNsIn = root->ns;
     if ( debug > 2 ){
+      using TiCC::operator <<;
       string dum;
       cerr << "root = " << Name( root ) << endl;
       cerr << "in namespace " << getNS( root, dum ) << endl;
-      cerr << "namespace list" << getNSlist( root ) << endl;
+      cerr << "namespace list" << getNSvalues( root ) << endl;
     }
     FoliaElement *result = 0;
     if ( root  ){
@@ -507,7 +598,11 @@ namespace folia {
 	     checkNS( root, NSFOLIA ) ){
 	  result = parseFoliaDoc( root );
 	  if ( result ){
-	    _id = result->id();
+	    if ( isNCName( result->id() ) )
+	      _id = result->id();
+	    else {
+	      // isNCName throws
+	    }
 	  }
 	}
 	else if ( Name( root ) == "DCOI" &&
@@ -533,26 +628,44 @@ namespace folia {
     KWargs kw = getArgs( args );
     string a = kw["annotator"];
     string t = kw["annotatortype"];
+    string d = kw["datetime"];
     kw.erase("annotator");
     kw.erase("annotatortype");
+    kw.erase("datetime");
     if ( kw.size() != 0 ){
-      throw XmlError( "declaration: expected 'annotator' of 'annotatortype', got '" + kw.begin()->first + "'" );
+      throw XmlError( "declaration: expected 'annotator', 'annotatortype' or 'datetime', got '" + kw.begin()->first + "'" );
     }
-    declare( type, st, a, t );
+    declare( type, st, a, t, d );
   }
 
+  string getNow() {
+    time_t Time;
+    time(&Time);
+    tm curtime;
+    localtime_r(&Time,&curtime);
+    char buf[256];
+    strftime( buf, 100, "%Y-%m-%dT%X", &curtime );
+    string res = buf;
+    return res;
+  }
+  
   void Document::declare( AnnotationType::AnnotationType type, 
-			  const string& s, const string& a, const string& t ){
-    if ( !isDeclared( type, s, a, t ) ){
-      annotationdefaults[type].insert( make_pair(s, at_t(a,t) ) );
-      //    cerr << "inserted [" << type << "][" << st << "](" << a << "," << t << ")" << endl;
+			  const string& s, const string& a, 
+			  const string& t, const string& ds ){
+    string d = ds;
+    if ( d == "now()" ){
+      d = getNow();
+    }
+    if ( !isDeclared( type, s, a, t, d ) ){
+      annotationdefaults[type].insert( make_pair(s, at_t(a,t,d) ) );
+      //    cerr << "inserted [" << type << "][" << st << "](" << a << "," << t << "," << d ")" << endl;
       //    cerr << "annotation defaults now: " <<  annotationdefaults << endl;
       
     }
   }
 
   FoliaElement* Document::addNode( ElementType et, const KWargs& kwargs ){
-    FoliaElement *res = FoliaElement::createElement( this, toString(et) );
+    FoliaElement *res = FoliaElement::createElement( this, et );
     res->setAttributes( kwargs );
     foliadoc->append( res );
     return res;
@@ -561,14 +674,18 @@ namespace folia {
   bool Document::isDeclared( AnnotationType::AnnotationType type,
 			     const string& s, 
 			     const string& a,
-			     const string& t ){
+			     const string& t,
+			     const string& d ){
+    if ( type == AnnotationType::NO_ANN ){
+      return true;
+    }
     map<AnnotationType::AnnotationType,multimap<string,at_t> >::const_iterator mit1 = annotationdefaults.find(type);
     if ( mit1 != annotationdefaults.end() ){
       if ( s.empty() )
 	throw runtime_error("isDeclared with empty set.");
       multimap<string,at_t>::const_iterator mit2 = mit1->second.lower_bound(s);
       while ( mit2 != mit1->second.upper_bound(s) ){
-	if ( mit2->second.a == a && mit2->second.t == t )
+	if ( mit2->second.a == a && mit2->second.t == t && mit2->second.d == d )
 	  return true;
 	++mit2;
       }
@@ -578,6 +695,9 @@ namespace folia {
 
   bool Document::isDeclared( AnnotationType::AnnotationType type,
 			     const string& s ){
+    if ( type == AnnotationType::NO_ANN ){
+      return true;
+    }
     map<AnnotationType::AnnotationType,multimap<string,at_t> >::const_iterator mit1 = annotationdefaults.find(type);
     if ( mit1 != annotationdefaults.end() ){
       if ( s.empty() )
@@ -589,6 +709,8 @@ namespace folia {
   }
 
   string Document::defaultset( AnnotationType::AnnotationType type ) const {
+    if ( type == AnnotationType::NO_ANN )
+      return "";
     // search a set. it must be unique. Otherwise return ""
     //    cerr << "zoek '" << type << "' default set " <<  annotationdefaults << endl;
     map<AnnotationType::AnnotationType,multimap<string,at_t> >::const_iterator mit1 = annotationdefaults.find(type);
@@ -604,6 +726,9 @@ namespace folia {
 
   std::string Document::defaultannotator( AnnotationType::AnnotationType type,
 					  const string& st ) const {
+    if ( type == AnnotationType::NO_ANN ){
+      return "";
+    }
     // if ( !st.empty() ){
     //   cerr << "zoek '" << st << "' default annotator " <<  annotationdefaults << endl;
     // }
@@ -630,6 +755,9 @@ namespace folia {
 
   std::string Document::defaultannotatortype( AnnotationType::AnnotationType type,
 					      const string& st ) const {
+    if ( type == AnnotationType::NO_ANN ){
+      return "";
+    }
     map<AnnotationType::AnnotationType,multimap<string,at_t> >::const_iterator mit1 = annotationdefaults.find(type);
     string result;
     if ( mit1 != annotationdefaults.end() ){
@@ -649,6 +777,27 @@ namespace folia {
     return result;
   }
 
+  std::string Document::defaultdatetime( AnnotationType::AnnotationType type,
+					 const string& st ) const {
+    map<AnnotationType::AnnotationType,multimap<string,at_t> >::const_iterator mit1 = annotationdefaults.find(type);
+    string result;
+    if ( mit1 != annotationdefaults.end() ){
+      if ( st.empty() ){
+	if ( mit1->second.size() == 1 )
+	  result = mit1->second.begin()->second.d;
+	return result;
+      }
+      else {
+	if ( mit1->second.count( st ) == 1 ){
+	  map<string,at_t>::const_iterator mit2 = mit1->second.find( st );
+	  result = mit2->second.d;
+	}
+      }
+    }
+    //  cerr << "get default ==> " << result << endl;
+    return result;
+  }
+
   void Document::setannotations( xmlNode *node ) const {
     map<AnnotationType::AnnotationType,multimap<string,at_t> >::const_iterator mit = annotationdefaults.begin();
     while ( mit != annotationdefaults.end() ){
@@ -657,7 +806,7 @@ namespace folia {
 	// Find the 'label' 
 	string label = toString( mit->first );
 	label += "-annotation";
-	xmlNode *n = xmlAddChild( node, newXMLNode( _foliaNs, label ) );
+	xmlNode *n = XmlNewNode( foliaNs(), label );
 	KWargs args;
 	string s = it->second.a;
 	if ( !s.empty() )
@@ -665,10 +814,14 @@ namespace folia {
 	s = it->second.t;
 	if ( !s.empty() )
 	  args["annotatortype"] = s;
+	s = it->second.d;
+	if ( !s.empty() )
+	  args["datetime"] = s;
 	s = it->first;
 	if ( s != "undefined" ) // the default
 	  args["set"] = s;
 	addAttributes( n, args );
+	xmlAddChild( node, n );
 	++it;
       }
       ++mit;
@@ -695,26 +848,31 @@ namespace folia {
 	atts["license"] = _license;
       if ( !_publisher.empty() )
 	atts["publisher"] = _publisher;
+      addAttributes( node, atts );
     }
     else if ( _metadatatype == IMDI  ||
 	      _metadatatype == CMDI ){
-      xmlAddChild( node, xmlCopyNodeList(metadata) );
       if ( !_metadatafile.empty() )
 	atts["src"] = _metadatafile;
+      addAttributes( node, atts );
+      xmlAddChild( node, xmlCopyNodeList(metadata) );
     }
-    addAttributes( node, atts );
   }
 
   void Document::setstyles( xmlDoc* doc ) const {
-    for ( size_t i=0; i < styles.size(); ++i ){
+    multimap<string,string>::const_iterator it = styles.begin();
+    while ( it != styles.end() ){
+      string content = "type=\"" + it->first 
+	+ "\" href=\"" + it->second + "\"";
       xmlAddChild( (xmlNode*)doc,
 		   xmlNewDocPI( doc,
 				(const xmlChar*)"xml-stylesheet", 
-				(const xmlChar*)styles[i].c_str() ) );
+				(const xmlChar*)content.c_str() ) );
+      ++it;
     }
   }
 
-  string Document::toXml( const string& nsLabel ) const {
+  string Document::toXml( const string& nsLabel, bool kanon ) const {
     string result;
     if ( foliadoc ){
       xmlDoc *outDoc = xmlNewDoc( (const xmlChar*)"1.0" );
@@ -724,15 +882,18 @@ namespace folia {
       xmlNs *xl = xmlNewNs( root, (const xmlChar *)"http://www.w3.org/1999/xlink", 
 			    (const xmlChar *)"xlink" );
       xmlSetNs( root, xl );
-      if ( !_foliaNs ){
+      if ( !_foliaNsIn ){
 	if ( nsLabel.empty() )
-	  _foliaNs = xmlNewNs( root, (const xmlChar *)NSFOLIA.c_str(), 0 );
+	  _foliaNsOut = xmlNewNs( root, (const xmlChar *)NSFOLIA.c_str(), 0 );
 	else
-	  _foliaNs = xmlNewNs( root,
-			       (const xmlChar *)NSFOLIA.c_str(),
-			       (const xmlChar*)nsLabel.c_str() );
+	  _foliaNsOut = xmlNewNs( root,
+				  (const xmlChar *)NSFOLIA.c_str(),
+				  (const xmlChar*)nsLabel.c_str() );
       }
-      xmlSetNs( root, _foliaNs );
+      else {
+	_foliaNsOut = xmlNewNs( root, _foliaNsIn->href, _foliaNsIn->prefix );
+      }
+      xmlSetNs( root, _foliaNsOut );
       KWargs attribs;
       attribs["_id"] = foliadoc->id(); // sort "id" in front!
       attribs["generator"] = string("libfolia-v") + VERSION;
@@ -740,21 +901,21 @@ namespace folia {
 	attribs["version"] = version;
       addAttributes( root, attribs );
 
-      xmlNode *md = xmlAddChild( root,  newXMLNode( _foliaNs, "metadata" ) );  
-      xmlNode *an = xmlAddChild( md,  newXMLNode( _foliaNs, "annotations" ) );
+      xmlNode *md = xmlAddChild( root, XmlNewNode( foliaNs(), "metadata" ) );  
+      xmlNode *an = xmlAddChild( md, XmlNewNode( foliaNs(), "annotations" ) );
       setannotations( an );
       setmetadata( md );
       vector<FoliaElement*>::const_iterator it= foliadoc->data.begin();
       while ( it != foliadoc->data.end() ){
-	xmlAddChild( root, (*it)->xml( true ) );
+	xmlAddChild( root, (*it)->xml( true, kanon ) );
 	++it;
       }
       xmlChar *buf; int size;
-      xmlDocDumpFormatMemory( outDoc, &buf, &size, 1 );
+      xmlDocDumpFormatMemoryEnc( outDoc, &buf, &size, "UTF-8", 1 );
       result = string( (const char *)buf, size );
       xmlFree( buf );
       xmlFreeDoc( outDoc );
-      _foliaNs = 0;
+      _foliaNsOut = 0;
     }
     else
       throw runtime_error( "can't save, no doc" );
@@ -985,6 +1146,12 @@ namespace folia {
     }
   }
 
+  inline std::ostream& operator<<( std::ostream& os, const Pattern& p ){
+    using TiCC::operator <<;
+    os << "pattern: " << p.sequence;
+    return os;
+  }
+
   bool Pattern::match( const UnicodeString& us, size_t& pos, int& gap, 
 		       bool& done, bool& flag ) const {
     UnicodeString s = us;
