@@ -1,8 +1,8 @@
 /*
-  $Id: foliautils.cxx 14316 2012-02-23 10:04:11Z sloot $
+  $Id: foliautils.cxx 15909 2013-04-03 13:52:24Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/libfolia/trunk/src/foliautils.cxx $
 
-  Copyright (c) 1998 - 2012
+  Copyright (c) 1998 - 2013
   ILK   - Tilburg University
   CLiPS - University of Antwerp
  
@@ -36,10 +36,12 @@
 #include <list>
 #include <stdexcept>
 #include <algorithm> 
-#include "folia/document.h"
-#include "folia/folia.h"
+#include "ticcutils/StringOps.h"
+#include "ticcutils/XMLtools.h"
+#include "libfolia/document.h"
 
 using namespace std;
+using namespace TiCC;
 
 namespace folia {
 
@@ -76,24 +78,39 @@ namespace folia {
     case AnnotationType::TEXT:
       result = "text";
       break; 
+    case AnnotationType::STRING:
+      result = "string";
+      break; 
     case AnnotationType::TOKEN:
       result = "token";
       break; 
-    case AnnotationType::GAP:
-      result = "gap";
-      break; 
     case AnnotationType::DIVISION:
       result = "div";
       break; 
+    case AnnotationType::PARAGRAPH: 
+      result = "paragraph";
+      break; 
+    case AnnotationType::LIST: 
+      result = "list";
+      break; 
+    case AnnotationType::FIGURE: 
+      result = "figure";
+      break; 
+    case AnnotationType::WHITESPACE: 
+      result = "whitespace";
+      break; 
+    case AnnotationType::LINEBREAK: 
+      result = "linebreak";
+      break; 
+    case AnnotationType::SENTENCE: 
+      result = "sentence";
+      break; 
     case AnnotationType::POS: 
       result = "pos";
       break; 
     case AnnotationType::LEMMA:
       result = "lemma";
       break; 
-    case AnnotationType::EVENT:
-      result = "event";
-      break; 
     case AnnotationType::DOMEIN:
       result = "domain";
       break; 
@@ -109,12 +126,12 @@ namespace folia {
     case AnnotationType::ENTITY: 
       result = "entity";
       break;
-    case AnnotationType::SUBENTITY: 
-      result = "subentity";
-      break;
     case AnnotationType::CORRECTION: 
       result = "correction";
       break;
+    case AnnotationType::SUGGESTION: 
+      result = "suggestion";
+      break;
     case AnnotationType::ERRORDETECTION: 
       result = "errordetection";
       break;
@@ -130,49 +147,67 @@ namespace folia {
     case AnnotationType::MORPHOLOGICAL:
       result = "morphological";
       break;
+    case AnnotationType::SUBENTITY: 
+      result = "subentity";
+      break;
+    case AnnotationType::EVENT:
+      result = "event";
+      break; 
     case AnnotationType::DEPENDENCY:
       result = "dependency";
       break;
     case AnnotationType::TIMEDEVENT:
-      result = "timedevent";
+      result = "timesegment";
       break;
-    default:
-      throw ValueError( " unknown translation for annotation: " + 
-			folia::toString(int(at)) );
+    case AnnotationType::GAP:
+      result = "gap";
+      break; 
+    case AnnotationType::ALIGNMENT:
+      result = "alignment";
+      break; 
+    case AnnotationType::COMPLEXALIGNMENT:
+      result = "complexalignment";
+      break; 
+    case AnnotationType::COREFERENCE:
+      result = "coreference";
+      break; 
+    case AnnotationType::SEMROLE:
+      result = "semrole";
+      break; 
+    case AnnotationType::METRIC:
+      result = "metric";
+      break; 
+    case AnnotationType::LAST_ANN:
+      throw logic_error("LAST_ANN");
     };
     return result;
   }
-
-  int to_lower( const int& i ){ return tolower(i); }
-  int to_upper( const int& i ){ return toupper(i); }
-
-  string lowercase( const string& in ){
-    string s = in;
-    transform( s.begin(), s.end(), s.begin(), to_lower );
-    return s;
-  }
-
-  string uppercase( const string& in ){
-    string s = in;
-    transform( s.begin(), s.end(), s.begin(), to_upper );
-    return s;
-  }
-
+ 
   AnnotationType::AnnotationType stringToAT( const string& at ){
     if ( at == "text" )
       return AnnotationType::TEXT;
+    if ( at == "string" )
+      return AnnotationType::STRING;
     if ( at == "token" )
       return AnnotationType::TOKEN;
     if ( at == "div" )
       return AnnotationType::DIVISION;
-    if ( at == "gap" )
-      return AnnotationType::GAP;
+    if ( at == "paragraph" )
+      return AnnotationType::PARAGRAPH;
+    if ( at == "list" )
+      return AnnotationType::LIST;
+    if ( at == "figure" )
+      return AnnotationType::FIGURE;
+    if ( at == "whitespace" )
+      return AnnotationType::WHITESPACE;
+    if ( at == "linebreak" )
+      return AnnotationType::LINEBREAK;
+    if ( at == "sentence" )
+      return AnnotationType::SENTENCE;
     if ( at == "pos" )
       return AnnotationType::POS;
     if ( at == "lemma" )
       return AnnotationType::LEMMA;
-    if ( at == "event" )
-      return AnnotationType::EVENT;
     if ( at == "domain" )
       return AnnotationType::DOMEIN;
     if ( at == "sense" )
@@ -183,10 +218,10 @@ namespace folia {
       return AnnotationType::CHUNKING;
     if ( at == "entity" )
       return AnnotationType::ENTITY;
-    if ( at == "subentity" )
-      return AnnotationType::SUBENTITY;
     if ( at == "correction" )
       return AnnotationType::CORRECTION; 
+    if ( at == "suggestion" )
+      return AnnotationType::SUGGESTION; 
     if ( at == "errordetection" )
       return AnnotationType::ERRORDETECTION; 
     if ( at == "alternative" )
@@ -197,10 +232,26 @@ namespace folia {
       return AnnotationType::SUBJECTIVITY;
     if ( at == "morphological" )
       return AnnotationType::MORPHOLOGICAL;
+    if ( at == "subentity" )
+      return AnnotationType::SUBENTITY;
+    if ( at == "event" )
+      return AnnotationType::EVENT;
     if ( at == "dependency" )
       return AnnotationType::DEPENDENCY;
-    if ( at == "timedevent" )
+    if ( at == "timesegment" )
       return AnnotationType::TIMEDEVENT;
+    if ( at == "gap" )
+      return AnnotationType::GAP;
+    if ( at == "complexalignment" )
+      return AnnotationType::COMPLEXALIGNMENT;
+    if ( at == "alignment" )
+      return AnnotationType::ALIGNMENT;
+    if ( at == "semrole" )
+      return AnnotationType::SEMROLE;
+    if ( at == "coreference" )
+      return AnnotationType::COREFERENCE;
+    if ( at == "metric" )
+      return AnnotationType::METRIC;
     throw ValueError( " unknown translation for attribute: " + at );
   }
 
@@ -211,12 +262,13 @@ namespace folia {
     case TextContent_t: result = "t"; break;
     case Text_t: result = "text"; break;
     case Event_t: result = "event"; break;
-    case TimedEvent_t: result = "timedevent"; break;
-    case Timings_t: result = "timings"; break;
+    case TimeSegment_t: result = "timesegment"; break;
+    case TimingLayer_t: result = "timing"; break;
     case LineBreak_t: result = "br"; break;
     case WhiteSpace_t: result = "whitespace"; break;
-    case Word_t: result = "word"; break;
+    case Word_t: result = "w"; break;
     case WordReference_t: result = "wref"; break; 
+    case Str_t: result = "str"; break;
     case Sentence_t: result = "s"; break;
     case Paragraph_t: result = "p"; break;
     case Division_t: result = "div"; break;
@@ -234,15 +286,20 @@ namespace folia {
     case Sense_t: result = "sense"; break; 
     case Subjectivity_t: result = "subjectivity"; break; 
     case Correction_t: result = "correction"; break;
-    case Annolay_t: result = "annotationlayer"; break; 
+    case AnnotationLayer_t: result = "annotationlayer"; break; 
     case SyntacticUnit_t: result = "su"; break; 
     case SyntaxLayer_t: result = "syntax"; break; 
     case Chunk_t: result = "chunk"; break; 
     case Chunking_t: result = "chunking"; break; 
     case Entity_t: result = "entity"; break;
     case Entities_t: result = "entities"; break;
-    case Subentity_t: result = "subentity"; break;
-    case Subentities_t: result = "subentities"; break;
+    case Semroles_t: result = "semroles"; break;
+    case Semrole_t: result = "semrole"; break;
+    case Coreferences_t: result = "coreferences"; break;
+    case CoreferenceLink_t: result = "coreferencelink"; break;
+    case CoreferenceChain_t: result = "coreferencechain"; break;
+    // case Subentity_t: result = "subentity"; break;
+    // case Subentities_t: result = "subentities"; break;
     case Morphology_t: result = "morphology"; break;
     case Morpheme_t: result = "morpheme"; break;
     case ErrorDetection_t: result = "errordetection"; break;
@@ -250,242 +307,429 @@ namespace folia {
     case Original_t: result = "original"; break;
     case Current_t: result = "current"; break;
     case Suggestion_t: result = "suggestion"; break;
-    case Alternative_t: result = "alternative"; break; 
-    case AltLayers_t: result = "altlayers"; break;
+    case Alternative_t: result = "alt"; break; 
+    case Alternatives_t: result = "altlayers"; break;
     case Description_t: result = "desc"; break;
     case Gap_t: result = "gap"; break;
     case Content_t: result = "content"; break;
-    case Feature_t: result = "feature"; break;
+    case Metric_t: result = "metric"; break;
+    case Feature_t: result = "feat"; break;
     case SynsetFeature_t: result = "synset"; break;
     case ActorFeature_t: result = "actor"; break;
-    case HeadFeature_t: result = "headfeat"; break;
-    case BegindatetimeFeature_t: result = "begindatetime"; break;
-    case EnddatetimeFeature_t: result = "enddatetime"; break;
+    case HeadFeature_t: result = "headfeature"; break;
+    case ValueFeature_t: result = "value"; break;
+    case TimeFeature_t: result = "time"; break;
+    case ModalityFeature_t: result = "modality"; break;
+    case LevelFeature_t: result = "level"; break;
+    case FunctionFeature_t: result = "function"; break;
+    case BeginDateTimeFeature_t: result = "begindatetime"; break;
+    case EndDateTimeFeature_t: result = "enddatetime"; break;
     case PlaceHolder_t: result = "placeholder"; break;
     case Dependencies_t: result = "dependencies"; break;
     case Dependency_t: result = "dependency"; break;
     case DependencyDependent_t: result = "dep"; break;
-    case DependencyHead_t: result = "hd"; break;
+    case Headwords_t: result = "hd"; break;
+    case Alignment_t: result = "alignment"; break;
+    case AlignReference_t: result = "aref"; break; 
     default:
       result = "Unknown Elementtype " + folia::toString( int(et) );
     }
     return result;
   }
 
-  FoliaElement *FoliaElement::createElement( Document *doc, 
-					     const string& tag ){
-    //factory;
+  ElementType stringToET( const string& tag ){
     if ( tag == "FoLiA" ){
-      return new FoLiA( doc );
-    }
-    if ( tag == "DCOI" ){
-      return new DCOI( doc );
+      return BASE;
     }
     if ( tag == "text" ){
-      return new Text( doc );
-    }
-    if ( tag == "event" ){
-      return new Event( doc );
-    }
-    if ( tag == "timedevent" ){
-      return new TimedEvent( doc );
-    }
-    if ( tag == "timings" ){
-      return new TimingLayer( doc );
-    }
-    if ( tag == "s" ){
-      return new Sentence( doc );
-    }
-    if ( tag == "t" ){
-      return new TextContent( doc );
-    }
-    if ( tag == "br" ){
-      return new LineBreak( doc );
-    }
-    if ( tag == "whitespace" ){
-      return new WhiteSpace( doc );
+      return Text_t;
     }
     if ( tag == "w" ){
-      return new Word( doc );
+      return Word_t;
+    }
+    if ( tag == "str" ){
+      return Str_t;
+    }
+    if ( tag == "event" ){
+      return Event_t;
+    }
+    if ( tag == "timesegment" ){
+      return TimeSegment_t;
+    }
+    if ( tag == "timing" ){
+      return TimingLayer_t;
+    }
+    if ( tag == "s" ){
+      return Sentence_t;
+    }
+    if ( tag == "t" ){
+      return TextContent_t;
+    }
+    if ( tag == "br" ){
+      return LineBreak_t;
+    }
+    if ( tag == "whitespace" ){
+      return WhiteSpace_t;
     }
     if ( tag == "figure" ){
-      return new Figure( doc );
+      return Figure_t;
     }
     if ( tag == "caption" ){
-      return new Caption( doc );
+      return Caption_t;
     }
     if ( tag == "label" ){
-      return new Label( doc );
+      return Label_t;
     }
     if ( tag == "list" ){
-      return new List( doc );
+      return List_t;
     }
     if ( tag == "listitem" ){
-      return new ListItem( doc );
+      return ListItem_t;
     }
     if ( tag == "p" ){
-      return new Paragraph( doc );
+      return Paragraph_t;
     }
     if ( tag == "new" ){
-      return new NewElement( doc );
+      return New_t;
     }
     if ( tag == "original" ){
-      return new Original( doc );
+      return Original_t;
+    }
+    if ( tag == "current" ){
+      return Current_t;
     }
     if ( tag == "suggestion" ){
-      return new Suggestion( doc );
+      return Suggestion_t;
     }
     if ( tag == "head" ){
-      return new Head( doc );
+      return Head_t;
     }
     if ( tag == "desc" ){
-      return new Description( doc );
+      return Description_t;
     }
     if ( tag == "gap" ){
-      return new Gap( doc );
+      return Gap_t;
     }
     if ( tag == "content" ){
-      return new Content( doc );
+      return Content_t;
+    }
+    if ( tag == "metric" ){
+      return Metric_t;
     }
     if ( tag == "div" ){
-      return new Division( doc );
+      return Division_t;
+    }
+    if ( tag == "annotationlayer" ){
+      return AnnotationLayer_t;
     }
     if ( tag == "pos" ){
-      return new PosAnnotation( doc );
+      return Pos_t;
     }
     if ( tag == "lemma" ){
-      return new LemmaAnnotation( doc );
+      return Lemma_t;
     }
     if ( tag == "phon" ){
-      return new PhonAnnotation( doc );
+      return Phon_t;
     }
     if ( tag == "domain" ){
-      return new DomainAnnotation( doc );
+      return Domain_t;
     }
     if ( tag == "sense" ){
-      return new SenseAnnotation( doc );
+      return Sense_t;
     }
     if ( tag == "syntax" ){
-      return new SyntaxLayer( doc );
+      return SyntaxLayer_t;
     }
     if ( tag == "subjectivity" ){
-      return new SubjectivityAnnotation( doc );
+      return Subjectivity_t;
     }
     if ( tag == "chunk" ){
-      return new Chunk( doc );
+      return Chunk_t;
     }
     if ( tag == "chunking" ){
-      return new ChunkingLayer( doc );
+      return Chunking_t;
     }
     if ( tag == "entity" ){
-      return new Entity( doc );
+      return Entity_t;
     }
     if ( tag == "entities" ){
-      return new EntitiesLayer( doc );
+      return Entities_t;
     }
-    if ( tag == "subentity" ){
-      return new Subentity( doc );
+    if ( tag == "semroles" ){
+      return Semroles_t;
     }
-    if ( tag == "subentities" ){
-      return new SubentitiesLayer( doc );
+    if ( tag == "semrole" ){
+      return Semrole_t;
+    }
+    if ( tag == "coreferences" ){
+      return Coreferences_t;
+    }
+    if ( tag == "coreferencelink" ){
+      return CoreferenceLink_t;
+    }
+    if ( tag == "coreferencechain" ){
+      return CoreferenceChain_t;
+    }
+    // if ( tag == "subentity" ){
+    //   return Subentity_t;
+    // }
+    // if ( tag == "subentities" ){
+    //   return Subentities_t;
+    // }
+    if ( tag == "alt" ){
+      return Alternative_t;
+    }
+    if ( tag == "placeholder" ){
+      return PlaceHolder_t;
+    }
+    if ( tag == "altlayers" ){
+      return Alternatives_t;
     }
     if ( tag == "su" ){
-      return new SyntacticUnit( doc );
+      return SyntacticUnit_t;
     }
     if ( tag == "wref" ){
-      return new WordReference( doc );
+      return WordReference_t;
     }
     if ( tag == "correction" ){
-      return new Correction( doc );
+      return Correction_t;
     }
     if ( tag == "errordetection" ){
-      return new ErrorDetection( doc );
+      return ErrorDetection_t;
     }
     if ( tag == "morphology" ){
-      return new MorphologyLayer( doc );
+      return Morphology_t;
     }
     if ( tag == "morpheme" ){
-      return new Morpheme( doc );
+      return Morpheme_t;
     }
     if ( tag == "feat" ){
-      return new Feature( doc );
+      return Feature_t;
     }
     if ( tag == "begindatetime" ){
-      return new BegindatetimeFeature( doc );
+      return BeginDateTimeFeature_t;
     }
     if ( tag == "enddatetime" ){
-      return new EnddatetimeFeature( doc );
+      return EndDateTimeFeature_t;
     }
     if ( tag == "synset" ){
-      return new SynsetFeature( doc );
+      return SynsetFeature_t;
     }
     if ( tag == "actor" ){
-      return new ActorFeature( doc );
+      return ActorFeature_t;
     }
-    if ( tag == "headfeat" ){
-      return new HeadFeature( doc );
+    if ( tag == "headfeature" ){
+      return HeadFeature_t;
+    }
+    if ( tag == "value" ){
+      return ValueFeature_t;
+    }
+    if ( tag == "time" ){
+      return TimeFeature_t;
+    }
+    if ( tag == "level" ){
+      return LevelFeature_t;
+    }
+    if ( tag == "function" ){
+      return FunctionFeature_t;
+    }
+    if ( tag == "modality" ){
+      return ModalityFeature_t;
     }
     if ( tag == "quote" ){
-      return new Quote( doc );
+      return Quote_t;
     }
     if ( tag == "dependencies" ){
-      return new DependenciesLayer( doc );
+      return Dependencies_t;
     }
     if ( tag == "dependency" ){
-      return new Dependency( doc );
+      return Dependency_t;
     }
     if ( tag == "dep" ){
-      return new DependencyDependent( doc );
+      return DependencyDependent_t;
     }
     if ( tag == "hd" ){
-      return new DependencyHead( doc );
+      return Headwords_t;
+    }
+    if ( tag == "alignment" ){
+      return Alignment_t;
+    }
+    if ( tag == "aref" ){
+      return AlignReference_t;
     }
     else {
-      //    throw runtime_error( "unknown tag <" + tag + ">" );
-      cerr << "unknown tag <" << tag << ">" << endl;
+      throw ValueError( "unknown tag <" + tag + ">" );
+    }
+    return BASE;
+  }
+
+  FoliaElement *FoliaElement::createElement( Document *doc, 
+					     ElementType et ){
+    switch ( et ){
+    case BASE:
+      return new FoLiA( doc );
+    case Text_t:
+      return new Text( doc );
+    case Word_t:
+      return new Word( doc );
+    case Str_t:
+      return new String( doc );
+    case Event_t:
+      return new Event( doc );
+    case TimeSegment_t:
+      return new TimeSegment( doc );
+    case TimingLayer_t:
+      return new TimingLayer( doc );
+    case Sentence_t:
+      return new Sentence( doc );
+    case TextContent_t:
+      return new TextContent( doc );
+    case LineBreak_t:
+      return new LineBreak( doc );
+    case WhiteSpace_t:
+      return new WhiteSpace( doc );
+    case Figure_t:
+      return new Figure( doc );
+    case Caption_t:
+      return new Caption( doc );
+    case Label_t:
+      return new Label( doc );
+    case List_t:
+      return new List( doc );
+    case ListItem_t:
+      return new ListItem( doc );
+    case Paragraph_t:
+      return new Paragraph( doc );
+    case New_t:
+      return new NewElement( doc );
+    case Original_t:
+      return new Original( doc );
+    case Current_t:
+      return new Current( doc );
+    case Suggestion_t:
+      return new Suggestion( doc );
+    case Head_t:
+      return new Head( doc );
+    case Description_t:
+      return new Description( doc );
+    case Gap_t:
+      return new Gap( doc );
+    case Content_t:
+      return new Content( doc );
+    case Metric_t:
+      return new MetricAnnotation( doc );
+    case Division_t:
+      return new Division( doc );
+    case AnnotationLayer_t:
+      return new AbstractAnnotationLayer( doc );
+    case Pos_t:
+      return new PosAnnotation( doc );
+    case Lemma_t:
+      return new LemmaAnnotation( doc );
+    case Phon_t:
+      return new PhonAnnotation( doc );
+    case Domain_t:
+      return new DomainAnnotation( doc );
+    case Sense_t:
+      return new SenseAnnotation( doc );
+    case SyntaxLayer_t:
+      return new SyntaxLayer( doc );
+    case Subjectivity_t:
+      return new SubjectivityAnnotation( doc );
+    case Chunk_t:
+      return new Chunk( doc );
+    case Chunking_t:
+      return new ChunkingLayer( doc );
+    case Entity_t:
+      return new Entity( doc );
+    case Entities_t:
+      return new EntitiesLayer( doc );
+    case Semroles_t:
+      return new SemanticRolesLayer( doc );
+    case Semrole_t:
+      return new SemanticRole( doc );
+    case Coreferences_t:
+      return new CoreferenceLayer( doc );
+    case CoreferenceLink_t:
+      return new CoreferenceLink( doc );
+    case CoreferenceChain_t:
+      return new CoreferenceChain( doc );
+    // case Subentity_t:
+    //   return new Subentity( doc );
+    // case Subentities_t:
+    //   return new SubentitiesLayer( doc );
+    case Alternative_t:
+      return new Alternative( doc );
+    case PlaceHolder_t:
+      return new PlaceHolder();
+    case Alternatives_t:
+      return new AlternativeLayers( doc );
+    case SyntacticUnit_t:
+      return new SyntacticUnit( doc );
+    case WordReference_t:
+      return new WordReference( doc );
+    case Correction_t:
+      return new Correction( doc );
+    case ErrorDetection_t:
+      return new ErrorDetection( doc );
+    case Morphology_t:
+      return new MorphologyLayer( doc );
+    case Morpheme_t:
+      return new Morpheme( doc );
+    case Feature_t:
+      return new Feature( doc );
+    case BeginDateTimeFeature_t:
+      return new BeginDateTimeFeature( doc );
+    case EndDateTimeFeature_t:
+      return new EndDateTimeFeature( doc );
+    case SynsetFeature_t:
+      return new SynsetFeature( doc );
+    case ActorFeature_t:
+      return new ActorFeature( doc );
+    case HeadFeature_t:
+      return new HeadFeature( doc );
+    case ValueFeature_t:
+      return new ValueFeature( doc );
+    case TimeFeature_t:
+      return new TimeFeature( doc );
+    case ModalityFeature_t:
+      return new ModalityFeature( doc );
+    case FunctionFeature_t:
+      return new FunctionFeature( doc );
+    case LevelFeature_t:
+      return new LevelFeature( doc );
+    case Quote_t:
+      return new Quote( doc );
+    case Dependencies_t:
+      return new DependenciesLayer( doc );
+    case Dependency_t:
+      return new Dependency( doc );
+    case DependencyDependent_t:
+      return new DependencyDependent( doc );
+    case Headwords_t:
+      return new Headwords( doc );
+    case Alignment_t:
+      return new Alignment( doc );
+    case AlignReference_t:
+      return new AlignReference( doc );
+    default:
+      throw ValueError( "unknown elementtype(" + toString(int(et)) + ")" );
     }
     return 0;
   }
 
-
-  string compress( const string& s ){
-    // remove leading and trailing spaces from a string
-    string result;
-    if ( !s.empty() ){
-      string::const_iterator b_it = s.begin();
-      while ( b_it != s.end() && isspace( *b_it ) ) ++b_it;
-      string::const_iterator e_it = s.end();
-      --e_it;
-      while ( e_it != s.begin() && isspace( *e_it ) ) --e_it;
-      if ( b_it <= e_it )
-	result = string( b_it, e_it+1 );
+  FoliaElement *FoliaElement::createElement( Document *doc, 
+					     const string& tag ){
+    
+    ElementType et = BASE;
+    try {
+      et = stringToET( tag );
     }
-    return result;
-  }
-
-  size_t split_at( const string& src, vector<string>& results, 
-		   const string& sep ){
-    // split a string into substrings, using seps as seperator
-    // silently skip empty entries (e.g. when two or more seperators co-incide)
-    // also purges trailing and leading spaces
-    results.clear();
-    string::size_type pos = 0;
-    string res;
-    while ( pos != string::npos ){
-      string::size_type p = src.find( sep, pos );
-      if ( p == string::npos ){
-	res = src.substr( pos );
-	pos = p;
-      }
-      else {
-	res = src.substr( pos, p - pos );
-	pos = p + sep.length();
-      }
-      res = folia::compress( res );
-      if ( !res.empty() )
-	results.push_back( res );
+    catch ( ValueError& e ){
+      cerr << e.what() << endl;
+      return 0;
     }
-    return results.size();
+    return createElement( doc, et );
   }
 
   KWargs getArgs( const std::string& s ){
@@ -592,10 +836,6 @@ namespace folia {
     return result;
   }
 
-  xmlNode *newXMLNode( xmlNs *ns,  const string& elem ){
-    return xmlNewNode( ns, (const xmlChar*)elem.c_str() );
-  }
-
   KWargs getAttributes( const xmlNode *node ){
     KWargs atts;
     if ( node ){
@@ -613,122 +853,20 @@ namespace folia {
     while ( it != attribs.end() ){
       //    cerr << "addAttributes(" << it->first << ", " << it->second << ")" << endl;
       if ( it->first == "_id" ){ // id is special
-	xmlNewNsProp( node, 0, XML_XML_ID,  (const xmlChar *)it->second.c_str() );
+	xmlSetProp( node, XML_XML_ID, (const xmlChar *)it->second.c_str() );
+      }
+      else if ( it->first == "lang" ){ // lang is special
+	xmlNodeSetLang( node, (const xmlChar*)it->second.c_str() );
       }
       else {
-	xmlNewNsProp( node, 0, 
-		      (const xmlChar*)it->first.c_str(), 
-		      (const xmlChar*)it->second.c_str() );
+	xmlSetProp( node,
+		    (const xmlChar*)it->first.c_str(), 
+		    (const xmlChar*)it->second.c_str() );
       }
       ++it;
     }
   }
 
-  string Name( const xmlNode *node ){
-    string result;
-    if ( node ){
-      result = (char *)node->name;
-    }
-    return result;
-  }
-
-  string XmlContent( const xmlNode *node ){
-    string result;
-    if ( node ){
-      xmlChar *tmp = xmlNodeListGetString( node->doc, node->children, 1 );
-      if ( tmp ){
-	result = string( (char *)tmp );
-	xmlFree( tmp );
-      }
-    }
-    return result;
-  }
-
-  string getNS( const xmlNode *node, string& prefix ){
-    string result;
-    prefix = "";
-    xmlNs *p = node->ns;
-    if ( p ){
-      if ( p->prefix ){
-	prefix = (char *)p->prefix;
-      }
-      result = (char *)p->href;
-    }
-    return result;
-  }
-
-  map<string,string> getNSlist( const xmlNode *node ){
-    map<string,string> result;
-    xmlNs *p = node->ns;
-    while ( p ){
-      string pre;
-      string val;
-      if ( p->prefix ){
-	pre = (char *)p->prefix;
-      }
-      val = (char *)p->href;
-      result[pre] = val;
-      p = p->next;
-    }
-    return result;
-  }
-
-  list<xmlNode*> FindLocal( xmlXPathContext* ctxt, 
-			    const string& xpath ){
-    list<xmlNode*> nodes;
-    xmlXPathObject* result = xmlXPathEval((xmlChar*)xpath.c_str(), ctxt);
-    if ( result ){
-      if (result->type != XPATH_NODESET) {
-	xmlXPathFreeObject(result);
-	throw runtime_error( "sorry, only nodeset result types supported for now." );
-	return nodes;
-      }
-      xmlNodeSet* nodeset = result->nodesetval;
-      if ( nodeset ){
-	for (int i = 0; i != nodeset->nodeNr; ++i)
-	  nodes.push_back(nodeset->nodeTab[i]);
-      }
-      else {
-	throw( runtime_error( "FindLocal: Missing nodeset" ) );
-      }
-      xmlXPathFreeObject(result);
-    }
-    else {
-      throw runtime_error( "Invalid Xpath: '" + xpath + "'" );
-    }
-    return nodes;
-  }
-
-  list<xmlNode*> FindNodes( xmlNode* node,
-			    const string& xpath ){
-    xmlXPathContext* ctxt = xmlXPathNewContext( node->doc );
-    ctxt->node = node;
-    ctxt->namespaces = xmlGetNsList( node->doc, ctxt->node );
-    ctxt->nsNr = 0;
-    if (ctxt->namespaces != 0 ) {
-      while (ctxt->namespaces[ctxt->nsNr] != 0 ){
-	ctxt->nsNr++;
-      }
-    }
-    list<xmlNode*> nodes = FindLocal( ctxt, xpath );
-    if (ctxt->namespaces != NULL)
-      xmlFree(ctxt->namespaces);
-    xmlXPathFreeContext(ctxt);
-    return nodes;
-  }
-
-  xmlNode *xPath( xmlNode *node, const string& xpath ){
-    // try to find a path, but it may not be there...
-    // if there are more, just return the first
-
-    list<xmlNode*> srch = FindNodes( node, xpath );
-    xmlNode *result = 0;
-    if ( srch.size() != 0 ){
-      result = srch.front();
-    }
-    return result;
-  }
-
   int toMonth( const string& ms ){
     int result = 0;
     try {
@@ -736,7 +874,7 @@ namespace folia {
       return result - 1;
     }
     catch( exception ){
-      string m = lowercase( ms );
+      string m = TiCC::lowercase( ms );
       if ( m == "jan" )
 	return 0;
       else if ( m == "feb" )
@@ -766,35 +904,37 @@ namespace folia {
     }
   }
 
-  tm *parseDate( const string& s ){
-    //  cerr << "try to read a date-time " << s << endl;
+  string parseDate( const string& s ){
+    if ( s.empty() )
+      return "";
+    //    cerr << "try to read a date-time " << s << endl;
     vector<string> date_time;
-    size_t num = split_at( s, date_time, "T");
+    size_t num = TiCC::split_at( s, date_time, "T");
     if ( num == 0 ){
-      num = split_at( s, date_time, " ");
+      num = TiCC::split_at( s, date_time, " ");
       if ( num == 0 ){
 	cerr << "failed to read a date-time " << s << endl;
 	return 0;
       }
     }
-    //  cerr << "found " << num << " parts" << endl;
+    //    cerr << "found " << num << " parts" << endl;
     tm *time = new tm();
     if ( num == 1 || num == 2 ){
-      //    cerr << "parse date " << date_time[0] << endl;
+      //      cerr << "parse date " << date_time[0] << endl;
       vector<string> date_parts;
-      size_t dnum = split_at( date_time[0], date_parts, "-" );
+      size_t dnum = TiCC::split_at( date_time[0], date_parts, "-" );
       switch ( dnum ){
       case 3: {
-	int year = stringTo<int>( date_parts[0] );
-	time->tm_year = year-1900;
+	int mday = stringTo<int>( date_parts[2] );
+	time->tm_mday = mday;
       }
       case 2: {
 	int mon = toMonth( date_parts[1] );
 	time->tm_mon = mon;
       }
       case 1: {
-	int mday = stringTo<int>( date_parts[2] );
-	time->tm_mday = mday;
+	int year = stringTo<int>( date_parts[0] );
+	time->tm_year = year-1900;
       }
 	break;
       default:
@@ -803,23 +943,24 @@ namespace folia {
       }
     }
     if ( num == 2 ){
-      //    cerr << "parse time " << date_time[1] << endl;
+      //      cerr << "parse time " << date_time[1] << endl;
       vector<string> date_parts;
-      num = split_at( date_time[1], date_parts, ":" );
+      num = TiCC::split_at( date_time[1], date_parts, ":" );
+      //      cerr << "parts " << date_parts << endl;
       switch ( num ){
       case 4:
 	// ignore
       case 3: {
-	int hour = stringTo<int>( date_parts[0] );
-	time->tm_hour = hour;
+	int sec = stringTo<int>( date_parts[2] );
+	time->tm_sec = sec;
       }
       case 2: {
 	int min = stringTo<int>( date_parts[1] );
 	time->tm_min = min;
       }
       case 1: {
-	int sec = stringTo<int>( date_parts[2] );
-	time->tm_sec = sec;
+	int hour = stringTo<int>( date_parts[0] );
+	time->tm_hour = hour;
       }
 	break;
       default:
@@ -827,11 +968,98 @@ namespace folia {
 	return 0;
       }
     }
-    //  cerr << "read _date time = ";
-    // char buf[100];
-    // strftime( buf, 100, "%Y-%b-%d %X", time );
-    // cerr << buf << endl;
-    return time;
+    // cerr << "read _date time = " << toString(time) << endl;
+    char buf[100];
+    strftime( buf, 100, "%Y-%m-%dT%X", time );
+    delete time;
+    return buf;
   }
 
+  bool AT_sanity_check(){
+    bool sane = true;
+    AnnotationType::AnnotationType at = AnnotationType::NO_ANN;
+    while ( ++at != AnnotationType::LAST_ANN ){
+      string s = toString( at );
+      if ( s.empty() ){
+	cerr << "no string translation for AnnotationType(" << at << ")" << endl;
+	sane = false;
+      }
+      else {
+	try {
+	  stringTo<AnnotationType::AnnotationType>( s );
+	}
+	catch ( ValueError& e ){
+	  cerr << "no AnnotationType found for string '" << s << "'" << endl;	
+	  sane = false;
+	}
+      }
+    }
+    return sane;
+  };
+
+  bool ET_sanity_check(){
+    bool sane = true;
+    ElementType et = BASE;
+    while ( ++et != LastElement ){
+      string s = toString( et );
+      if ( s.empty() ){
+	cerr << "no string translation for ElementType(" << et << ")" << endl;
+	sane = false;
+      }
+      else {
+	ElementType et2;
+	try {
+	  et2 = stringToET( s );
+	}
+	catch ( ValueError& e ){
+	  cerr << "no element type found for string '" << s << "'" << endl;
+	  sane = false;
+	  continue;
+	}
+	if ( et != et2 ){
+	  cerr << "Argl: toString(ET) doesn't match original:" << s 
+	       << " vs " << toString(et2) << endl;
+	  sane = false;
+	  continue;
+	}
+	FoliaElement *tmp = FoliaElement::createElement( 0, s );
+	if ( tmp == 0 ) {
+	  cerr << "no Element created found for string '" << s << "'" << endl;	
+	  sane = false;
+	}
+	else {
+	  if ( et != tmp->element_id() ){
+	    cerr << "the element type of " << tmp << " != " << et << endl;
+	    sane = false;
+	  }
+	  if ( s != tmp->xmltag() ){
+	    cerr << "the xmltag " << tmp->xmltag() << " != " << s << endl;
+	    sane = false;
+	  }
+	}
+      }
+    }
+    return sane;
+  };
+
+  bool isNCName( const std::string& s ){
+    static string extra=".-_";
+    if ( s.empty() )
+      throw XmlError( "an empty string is not a valid NCName." );
+    else if ( !isalpha(s[0]) )
+      throw XmlError( "'" 
+		      + s 
+		      + "' is not a valid NCName. (must start with character)." );
+    else {
+      for ( size_t i=1; i < s.length(); ++i ){
+	if ( !isalnum(s[i]) &&
+	     extra.find(s[i]) == string::npos )
+	  throw XmlError( "'" + s 
+			  + "' is not a valid NCName.(invalid '" 
+			  + char(s[i]) + "' found" );
+      }
+    }
+    return true;
+  }
+  
 } //namespace folia
